import { sep, resolve, dirname } from 'node:path';
import { createSyncFn, TsRunner } from 'synckit';
import { baseParse } from '@vue/compiler-core';

function normalizePath(path) {
  return path.split(sep).join("/");
}
function isUndefined(v) {
  return v === void 0 || v === null;
}

function getPropsMap(attrs) {
  const map = {};
  for (const { name, value, exp, arg } of attrs) {
    if (name === "bind") {
      if (!isUndefined(arg?.content))
        map[arg.content] = JSON.parse(exp.content);
      continue;
    }
    if (isUndefined(value?.content) || value?.content === "")
      map[name] = true;
    else if (["true", "false"].includes(value?.content || ""))
      map[name] = value?.content === "true";
    else
      map[name] = value?.content;
  }
  return map;
}
function parseProps(content) {
  const ast = baseParse(content);
  const demoElement = ast.children[0];
  return getPropsMap(demoElement.props);
}

function parseType(type) {
  return type?.name === "union" ? "`enum`" : `\`${format(type?.name)}\``;
}
function parseTypeExpand(type) {
  return type?.name === "union" ? `\`${type.elements?.map((v) => v.name).join(" 666 ")}\`` : `\`${format(type?.name)}\``;
}
function parseEventType(eventType) {
  return eventType?.names[0] === "union" ? `\`(${eventType.elements?.map((v) => v.name).join("666")}): void\`` : "`(): void`";
}
function format(type) {
  if (!type)
    return "-";
  return `\`${type}\``;
}

const parseFilePath = resolve(__dirname, "./parse.mjs");
const parse = createSyncFn(parseFilePath, {
  tsRunner: TsRunner.TSX
});
function popup(text, content) {
  if (text === content)
    return text;
  content = content.replace(/"/g, "'");
  content = content.replace(/`/g, "`");
  return `<div style="text-decoration: underline;cursor: pointer; padding: 0 2px;" title="${content}">${text}</div>`;
}
function use(md, { dir, ...opts } = {}) {
  function addRenderRule(type) {
    const defaultRender = md.renderer.rules[type];
    md.renderer.rules[type] = (tokens, idx, options, env, self) => {
      const token = tokens[idx];
      const content = token.content.trim();
      if (!content.match(/^<stories\s/))
        return defaultRender(tokens, idx, options, env, self);
      let path = env.path;
      const props = parseProps(content);
      if (!props.src) {
        console.error(`rendering ${path}: src prop is required`);
        return defaultRender(tokens, idx, options, env, self);
      }
      if (props.src.includes("~/") && dir)
        path = dir;
      const { src, title } = props;
      const dirPath = dirname(path);
      const srcPath = normalizePath(resolve(dirPath, src));
      const parsed = parse(srcPath);
      let text = "";
      function incTitle(key) {
        text += `## ${title ? `${title} ` : ""}${key}

`;
      }
      function incTableHead(...heads) {
        text += `| ${heads.join(" | ")} |
`;
        text += `| ${heads.map(() => "---").join(" | ")} |
`;
      }
      function incTableBody(...bodies) {
        text += `| ${bodies.join(" | ")} |
`;
      }
      function incTableEnd() {
        text += "\n";
      }
      if (parsed.props) {
        incTitle("Props");
        incTableHead("Name", "Description", "Type", "Default");
        for (const prop of parsed.props) {
          const type2 = (opts.popup || popup)(parseType(prop.type), parseTypeExpand(prop.type));
          incTableBody(prop.name, prop.description || "-", type2, format(prop.defaultValue?.value));
        }
        incTableEnd();
      }
      if (parsed.events) {
        incTitle("Events");
        incTableHead("Name", "Description", "Type");
        for (const event of parsed.events)
          incTableBody(event.name, event.description || "-", parseEventType(event.type));
        incTableEnd();
      }
      if (parsed.slots) {
        incTitle("Slots");
        incTableHead("Name", "Description");
        for (const slot of parsed.slots)
          incTableBody(slot.name, slot.description || "-");
        incTableEnd();
      }
      if (parsed.expose) {
        incTitle("Exposes");
        incTableHead("Name", "Description", "Type");
        for (const expose of parsed.expose) {
          const type2 = expose.tags?.find((v) => v.title === "type")?.type;
          incTableBody(expose.name, expose.description || "-", parseTypeExpand(type2));
        }
        incTableEnd();
      }
      return md.render(text.replace(/ #/g, "777")).replace(/666/g, "|").replace(/777/g, "#");
    };
  }
  addRenderRule("html_block");
  addRenderRule("html_inline");
}

export { use as default };
